SystemOrganization addCategory: #'BT-Utils'!

TestCase subclass: #BencodeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BT-Utils'!

!BencodeTest commentStamp: 'fn 7/13/2012 02:24' prior: 0!
Test cases for bencoding integers, strings, lists, and dictionaries.!

!BencodeTest methodsFor: 'testing' stamp: 'fn 7/13/2012 02:25'!
testBencodeDictionary

	| d |
	d := Dictionary new.
	
	self assert: (d bencode) = 'de'.
	
	d at: 'first' put: 'Faried'.
	d at: 'last' put: 'Nawaz'.
	d at: 42 put: 'forty-two'.
	d at: 'age-diff' put: #(0 2 5).
	
	self assert: (d bencode) = 'd4:last5:Nawaz8:age-diffli0ei2ei5ee5:first6:Fariedi42e9:forty-twoe'.! !

!BencodeTest methodsFor: 'testing' stamp: 'fn 7/13/2012 02:24'!
testBencodeInteger

	self assert: (123 bencode) = 'i123e'.
	self assert: (0 bencode) = 'i0e'.
! !

!BencodeTest methodsFor: 'testing' stamp: 'fn 7/13/2012 02:25'!
testBencodeList

	self assert: (#() bencode) = 'le'.
	self assert: (#(1 2 3) bencode) = 'li1ei2ei3ee'.
	self assert: (#(1 'abc' 3 'def') bencode) = 'li1e3:abci3e3:defe'.
	self assert: (#('abc' 42 #('c' 4 't' 5) 'cars') bencode) = 'l3:abci42el1:ci4e1:ti5ee4:carse'.


! !

!BencodeTest methodsFor: 'testing' stamp: 'fn 7/13/2012 02:25'!
testBencodeString

	self assert: ('teststring' bencode) = '10:teststring'.
	self assert: ('' bencode) = '0:'.
! !

!Dictionary methodsFor: '*bt-utils' stamp: 'fn 7/14/2012 00:22'!
bencode

	|buffer|
	
	buffer := WriteStream on: ''.
	self keysSortedSafely do: [ :k |
		k isString ifFalse: [^self error: 'dictionary key is not a string; cannot bencode.'].
		buffer nextPutAll: k bencode.
		buffer nextPutAll: (self at: k) bencode.
		].
		^'d', buffer contents, 'e'.! !

!Integer methodsFor: '*bt-utils' stamp: 'fn 7/13/2012 01:22'!
bencode

	^'i', self asString, 'e'! !

!SequenceableCollection methodsFor: '*bt-utils' stamp: 'fn 7/13/2012 01:39'!
bencode

	|buffer|
	
	buffer := WriteStream on:''.
	(self collect: [ :i | i bencode ]) asStringOn: buffer delimiter: ''.
	^'l', buffer contents, 'e'.! !

!String methodsFor: '*bt-utils' stamp: 'fn 7/16/2012 19:15'!
bdecode
	"Decode a bencoded string."
	self
		ifEmpty: [^ ''].
	^ (self bdecodeWorker: 1)
		at: 2! !

!String methodsFor: '*bt-utils' stamp: 'fn 7/16/2012 19:41'!
bdecodeDictionary: index 
	"Decode a bencoded dictionary."
	| d idx ret k |
	d := Dictionary new.
	idx := index + 1.
	k := nil.
	[(self at: idx)
		~= $e]
		whileTrue: [ret := self bdecodeWorker: idx.
			idx := ret at: 1.
			k isNil
				ifTrue: [k := ret at: 2]
				ifFalse: [d
						at: k
						put: (ret at: 2).
					k := nil]].
	^ Array with: idx with: d! !

!String methodsFor: '*bt-utils' stamp: 'fn 7/16/2012 19:39'!
bdecodeInteger: index 
	"Decode a bencoded integer."
	| end |
	end := self indexOf: $e startingAt: index.
	^ Array with: end + 1 with: (self copyFrom: index + 1 to: end - 1) asInteger! !

!String methodsFor: '*bt-utils' stamp: 'fn 7/16/2012 19:39'!
bdecodeList: index 
	"Decode a bencoded list."
	| ls idx ret |
	ls := OrderedCollection new.
	idx := index + 1.
	[(self at: idx)
		~= $e]
		whileTrue: [ret := self bdecodeWorker: idx.
			idx := ret at: 1.
			ls
				add: (ret at: 2)].
	^ Array with: idx with: ls asArray! !

!String methodsFor: '*bt-utils' stamp: 'fn 7/16/2012 19:40'!
bdecodeString: index 
	"Decode a bencoded string."
	| col len |
	col := self indexOf: $: startingAt: index.
	col = 0
		ifTrue: [^ ''].
	len := (self copyFrom: index to: col) asInteger.
	len < 0
		ifTrue: [^ ''].
	^ Array
		with: col + len + 1
		with: (self copyFrom: col + 1 to: col + len)! !

!String methodsFor: '*bt-utils' stamp: 'fn 7/16/2012 19:46'!
bdecodeWorker: index 
	| c |
	c := self at: index.
	c = $i
		ifTrue: [^ self bdecodeInteger: index].
	c isDigit
		ifTrue: [^ self bdecodeString: index].
	c = $l
		ifTrue: [^ self bdecodeList: index].
	c = $d
		ifTrue: [^ self bdecodeDictionary: index]! !

!String methodsFor: '*bt-utils' stamp: 'fn 7/13/2012 01:23'!
bencode

	^self size asString, ':', self.! !
